@@ -1,7 +1,11 @@
-/* init.c - MemTest-86  Version 3.1
+/* init.c - MemTest-86  Version 3.2
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady
+ * ----------------------------------------------------
+ * MemTest86+ V1.11 Specific code (GPL V2.0)
+ * By Samuel DEMEULEMEESTER, sdemeule@memtest.org
+ * http://www.x86-secret.com - http://www.memtest.org
  */
 
 #include "test.h"
@@ -18,6 +22,7 @@ struct cpu_ident cpu_id;
 ulong st_low, st_high;
 ulong end_low, end_high;
 ulong cal_low, cal_high;
+ulong extclock;
 
 static ulong memspeed(ulong src, ulong len, int iter);
 static void cpu_type(void);
@@ -46,7 +51,7 @@ static void display_init(void)
 	for(i=0, pp=(char *)(SCREEN_ADR+1); i<TITLE_WIDTH; i++, pp+=2) {
 		*pp = 0x47;
 	}
-	cprint(0, 0, "      Memtest-86 v3.1       ");
+	cprint(0, 0, "      Memtest-86 v3.2       ");
 
 	/* Do reverse video for the bottom display line */
 	for(i=0, pp=(char *)(SCREEN_ADR+1+(24 * 160)); i<80; i++, pp+=2) {
@@ -87,22 +92,22 @@ void init(void)
 	/* setup pci */
 	pci_init();
 
-	/* Find the memory controller */
-	find_controller();
-
 	v->test = 0;
 	v->testsel = -1;
 	v->msg_line = LINE_SCROLL-1;
 	v->scroll_start = v->msg_line * 160;
 
-	cprint(LINE_CPU+1, 0, "L1 Cache - Unknown");
-	cprint(LINE_CPU+2, 0, "L2 Cache - Unknown");
-	cprint(LINE_CPU+3, 0, "Memory ");
+	cprint(LINE_CPU+1, 0, "L1 Cache: Unknown ");
+	cprint(LINE_CPU+2, 0, "L2 Cache: Unknown ");
+	cprint(LINE_CPU+3, 0, "Memory  : ");
 	aprint(LINE_CPU+3, 10, v->test_pages);
-	cprint(LINE_CPU+4, 0, "Chipset ");
+	cprint(LINE_CPU+4, 0, "Chipset : ");
 
 	cpu_type();
 
+	/* Find the memory controller */
+	find_controller();
+
 	if (v->rdtsc) {
 		cacheable();
 		cprint(LINE_TIME, COL_TIME+4, ":  :");
@@ -113,11 +118,11 @@ void init(void)
 	cprint(3, COL_MID,"Testing: ");
 	cprint(4, COL_MID,"Pattern: ");
 	cprint(LINE_INFO-2, 0, " WallTime   Cached  RsvdMem   MemMap   Cache  ECC  Test  Pass  Errors ECC Errs");
-	cprint(LINE_INFO-1, 0, "----------  ------  -------  --------  -----  ---  ----  ----  ------ --------");
+	cprint(LINE_INFO-1, 0, " ---------  ------  -------  --------  -----  ---  ----  ----  ------ --------");
 	cprint(LINE_INFO, COL_TST, "Std");
 	cprint(LINE_INFO, COL_PASS, "    0");
 	cprint(LINE_INFO, COL_ERR, "     0");
-	cprint(LINE_INFO+1, 0, "------------------------------------------------------------------------------");
+	cprint(LINE_INFO+1, 0, " -----------------------------------------------------------------------------");
 
 	for(i=0; i < 5; i++) {
 		cprint(i, COL_MID-2, "| ");
@@ -252,7 +257,7 @@ unsigned long page_of(void *addr)
 /*
  * Find CPU type and cache sizes
  */
-static void cpu_type(void)
+void cpu_type(void)
 {
 	int i, off=0;
 	int l1_cache=0, l2_cache=0;
@@ -338,6 +343,9 @@ static void cpu_type(void)
 			case 14:
 				cprint(LINE_CPU, 0, "AMD 5x86-WT");
 				break;
+			case 15:
+				cprint(LINE_CPU, 0, "AMD 5x86-WB");
+				break;
 			}
 			/* Since we can't get CPU speed or cache info return */
 			return;
@@ -371,6 +379,14 @@ static void cpu_type(void)
 				l2_cache = (cpu_id.cache_info[11] << 8);
 				l2_cache += cpu_id.cache_info[10];
 				break;
+			case 13: 
+				cprint(LINE_CPU, 0, "AMD K6-III+"); 
+				off = 11; 
+				l1_cache = cpu_id.cache_info[3]; 
+				l1_cache += cpu_id.cache_info[7]; 
+				l2_cache = (cpu_id.cache_info[11] << 8); 
+				l2_cache += cpu_id.cache_info[10]; 
+				break;
 			}
 			break;
 		case 6:
@@ -382,16 +398,30 @@ static void cpu_type(void)
 
 			switch(cpu_id.model) {
 			case 1:
+				cprint(LINE_CPU, 0, "AMD Athlon (0.25)");
+				off = 17;
+				break;
 			case 2:
 			case 4:
+				cprint(LINE_CPU, 0, "AMD Athlon (0.18)");
+				off = 17;
+				break;
 			case 6:
 				if (l2_cache == 64) {
-					cprint(LINE_CPU, 0, "AMD Duron");
-					off = 9;
+					cprint(LINE_CPU, 0, "AMD Duron (0.18)");
 				} else {
-					cprint(LINE_CPU, 0, "AMD Athlon");
-					off = 10;
+					cprint(LINE_CPU, 0, "Athlon XP (0.18)");
 				}
+				off = 16;
+				break;
+			case 8:
+			case 10:
+				if (l2_cache == 64) {
+					cprint(LINE_CPU, 0, "AMD Duron (0.13)");
+				} else {
+					cprint(LINE_CPU, 0, "Athlon XP (0.13)");
+				}
+				off = 16;
 				break;
 			case 3:
 			case 7:
@@ -404,11 +434,6 @@ static void cpu_type(void)
 					l2_cache = 64;
 				}
 				break;
-			case 8:
-			case 9:
-			case 10:
-				cprint(LINE_CPU, 0, "Athlon XP/MP");
-				off = 12;
 			}
 			break;
 		case 15:
@@ -420,13 +445,25 @@ static void cpu_type(void)
 
 			switch(cpu_id.model) {
 			case 4:
-				cprint(LINE_CPU, 0, "Athlon 64");
-				off = 9;
+			case 12:
+				if (l2_cache == 256) {
+				    cprint(LINE_CPU, 0, "AMD Sempron (0.13)");
+				    off = 18;
+				} else {
+				    cprint(LINE_CPU, 0, "Athlon 64 (0.13)");
+				    off = 16;
+				}
 				break;
 			case 5:
-				cprint(LINE_CPU, 0, "AMD Opteron");
-				off = 11;
+				cprint(LINE_CPU, 0, "AMD Opteron (0.13)");
+				off = 18;
+				break;
+			default:
+				cprint(LINE_CPU, 0, "AMD Athlon 64");
+				off = 13;
+				break;
 			}
+			break;
 		}
 		break;
 
@@ -486,9 +523,12 @@ static void cpu_type(void)
 			case 0x8:
 			case 0xc:
 			case 0x67:
+			case 0x60:
 				l1_cache += 16;
 				break;
 			case 0x68:
+			case 0x2c:
+			case 0x30:
 				l1_cache += 32;
 				break;
 			case 0x40:
@@ -496,24 +536,30 @@ static void cpu_type(void)
 				break;
 			case 0x41:
 			case 0x79:
+			case 0x39:
+			case 0x3b:
 				l2_cache = 128;
 				break;
 			case 0x42:
 			case 0x7a:
 			case 0x82:
+			case 0x3c:
 				l2_cache = 256;
 				break;
 			case 0x43:
 			case 0x7b:
 			case 0x83:
+			case 0x86:
 				l2_cache = 512;
 				break;
 			case 0x44:
 			case 0x7c:
 			case 0x84:
+			case 0x87:
 				l2_cache = 1024;
 				break;
 			case 0x45:
+			case 0x7d:
 			case 0x85:
 				l2_cache = 2048;
 				break;
@@ -552,6 +598,7 @@ static void cpu_type(void)
 				off = 11;
 				break;
 			case 3:
+			case 4:
 				cprint(LINE_CPU, 0, "Pentium II");
 				off = 10;
 				break;
@@ -569,38 +616,149 @@ static void cpu_type(void)
 					cprint(LINE_CPU, 0, "Celeron");
 					off = 7;
 				} else {
+				    if (cpu_id.step == 0 || cpu_id.step == 5) {
+					cprint(LINE_CPU, 0, "Celeron-A");
+					off = 9;
+				    } else {
 					cprint(LINE_CPU, 0, "Pentium II");
 					off = 10;
+				    }
 				}
 				break;
 			case 7:
 			case 8:
 			case 10:
 			case 11:
-				cprint(LINE_CPU, 0, "Pentium III");
-				off = 11;
+				if (l2_cache == 128) {
+					cprint(LINE_CPU, 0, "Celeron");
+					off = 7;
+				} else {
+					cprint(LINE_CPU, 0, "Pentium III");
+					off = 11;
+				}
+				break;
+			case 9:
+				if (l2_cache == 512) {
+					cprint(LINE_CPU, 0, "Celeron M (0.13)");
+				} else {
+					cprint(LINE_CPU, 0, "Pentium M (0.13)");
+				}
+				off = 16;
+				break;
+			case 13:
+				if (l2_cache == 512) {
+					cprint(LINE_CPU, 0, "Celeron M (0.09)");
+				} else {
+					cprint(LINE_CPU, 0, "Pentium M (0.09)");
+				}
+				off = 16;
 				break;
 			}
 			break;
 		case 15:
-			cprint(LINE_CPU, 0, "Pentium 4");
-			off = 9;
+			switch(cpu_id.model) {
+			case 0:
+			case 1:			
+				if (l2_cache == 128) {
+					cprint(LINE_CPU, 0, "Celeron (0.18)");
+					off = 14;	
+				} else if (cpu_id.pwrcap == 0x0B) {
+					cprint(LINE_CPU, 0, "Xeon DP (0.18)");
+					off = 14;
+				} else if (cpu_id.pwrcap == 0x0C) {
+					cprint(LINE_CPU, 0, "Xeon MP (0.18)");
+					off = 14;
+				} else {
+					cprint(LINE_CPU, 0, "Pentium 4 (0.18)");
+					off = 16;
+				}
+				break;
+			case 2:
+				if (l2_cache == 128) {
+					cprint(LINE_CPU, 0, "Celeron (0.13)");
+					off = 14;	
+				} else if (cpu_id.pwrcap == 0x0B) {
+					cprint(LINE_CPU, 0, "Xeon DP (0.13)");
+					off = 14;
+				} else if (cpu_id.pwrcap == 0x0C) {
+					cprint(LINE_CPU, 0, "Xeon MP (0.13)");
+					off = 14;
+				} else {
+					cprint(LINE_CPU, 0, "Pentium 4 (0.13)");
+					off = 16;
+				}
+				break;
+			case 3:
+			case 4:
+				if (l2_cache == 256) {
+					cprint(LINE_CPU, 0, "Celeron (0.09)");
+					off = 14;	
+				} else if (cpu_id.pwrcap == 0x0B) {
+					cprint(LINE_CPU, 0, "Xeon DP (0.09)");
+					off = 14;
+				} else if (cpu_id.pwrcap == 0x0C) {
+					cprint(LINE_CPU, 0, "Xeon MP (0.09)");
+					off = 14;
+				} else {
+					cprint(LINE_CPU, 0, "Pentium 4 (0.09)");
+					off = 16;
+				}
+				break;
+			case 5:
+				cprint(LINE_CPU, 0, "Pentium 4");
+				off = 9;
+				break;
+			}
+			break;
+
 		}
 		break;
 
 	/* Cyrix Processors with CPUID */
 	case 'C':
-		switch(cpu_id.model) {
-		case 0:
-			cprint(LINE_CPU, 0, "Cyrix 6x86MX/MII");
-			off = 16;
+
+		switch(cpu_id.type) {
+		case 6: // VIA C3
+			switch(cpu_id.model) {
+			case 6:
+				cprint(LINE_CPU, 0, "Cyrix III");
+				off = 9;
+				break;
+			case 7:
+				if (cpu_id.step < 8) {
+				    cprint(LINE_CPU, 0, "VIA C3 Samuel2");
+				    off = 14;
+				} else {
+				    cprint(LINE_CPU, 0, "VIA C3 Ezra-T");
+				    off = 13;				
+				}	
+				break;
+			case 8:
+				cprint(LINE_CPU, 0, "VIA C3 Ezra-T");
+				off = 13;
+				break;
+			case 9:
+				cprint(LINE_CPU, 0, "VIA C3 Nehemiah");
+				off = 15;
 			break;
-		case 4:
-			cprint(LINE_CPU, 0, "Cyrix GXm");
-			off = 9;
+			}
+			// L1 = L2 = 64 KB from Cyrix III to Nehemiah
+			l1_cache = 64;
+			l2_cache = 64;
 			break;
+		case 5:
+			switch(cpu_id.model) {
+			case 0:
+				cprint(LINE_CPU, 0, "Cyrix 6x86MX/MII");
+				off = 16;
+				break;
+			case 4:
+				cprint(LINE_CPU, 0, "Cyrix GXm");
+				off = 9;
+				break;
+			}
+			return;
 		}
-		return;
 		break;
 
 	/* Unknown processor */
@@ -621,7 +779,7 @@ static void cpu_type(void)
 
 	/* Print CPU speed */
 	if ((speed = cpuspeed()) > 0) {
-		if (speed < 1000000) {
+		if (speed < 999500) {
 			speed += 50; /* for rounding */
 			cprint(LINE_CPU, off, "    . MHz");
 			dprint(LINE_CPU, off+1, speed/1000, 3, 1);
@@ -629,16 +787,17 @@ static void cpu_type(void)
 		} else {
 			speed += 500; /* for rounding */
 			cprint(LINE_CPU, off, "      Mhz");
-			dprint(LINE_CPU, off+1, speed/1000, 5, 0);
+			dprint(LINE_CPU, off, speed/1000, 5, 0);
 		}
+		extclock = speed;
 	}
 
 	/* Print out L1 cache info */
 	/* To measure L1 cache speed we use a block size that is 1/4th */
 	/* of the total L1 cache size since half of it is for instructions */
 	if (l1_cache) {
-		cprint(LINE_CPU+1, 9, "     K     ");
-		dprint(LINE_CPU+1, 10, l1_cache, 4, 0);
+		cprint(LINE_CPU+1, 10, "     K     ");
+		dprint(LINE_CPU+1, 11, l1_cache, 3, 0);
 		if ((speed=memspeed((ulong)mapping(0x100),
 				(l1_cache / 4) * 1024, 50))) {
 			cprint(LINE_CPU+1, 15, "      MB/s");
@@ -652,7 +811,7 @@ static void cpu_type(void)
 	/* cache is bigger than the L2 */
 	if (l2_cache) {
 		cprint(LINE_CPU+2, 9, "     K     ");
-		cprint(LINE_CPU+2, 0, "L2 Cache    ?K");
+		cprint(LINE_CPU+2, 0, "L2 Cache:    ?K");
 		dprint(LINE_CPU+2, 10, l2_cache, 4, 0);
 
 		if (l2_cache < l1_cache) {
