@@ -1,4 +1,4 @@
-/* lib.c - MemTest-86  Version 3.1
+/* lib.c - MemTest-86  Version 3.2
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady
@@ -402,19 +402,11 @@ void set_cache(int val)
 	switch(val) {
 	case 0:
 		cache_off();	
-		if (v->cache_flag) {
-			cprint(LINE_INFO, COL_CACHE, "OFF");
-		} else {
-			cprint(LINE_INFO, COL_CACHE, "off");
-		}
+		cprint(LINE_INFO, COL_CACHE, "off");
 		break;
 	case 1:
 		cache_on();
-		if (v->cache_flag) {
-			cprint(LINE_INFO, COL_CACHE, " ON");
-		} else {
-			cprint(LINE_INFO, COL_CACHE, " on");
-		}
+		cprint(LINE_INFO, COL_CACHE, " on");
 		break;
 	}
 }
@@ -892,61 +884,3 @@ void wait_keyup( void ) {
 		}
 	}
 }
-#ifdef LP
-#define DATA            0x00
-#define STATUS          0x01
-#define CONTROL         0x02
-
-#define LP_PBUSY        0x80  /* inverted input, active high */
-#define LP_PERRORP      0x08  /* unchanged input, active low */
-
-#define LP_PSELECP      0x08  /* inverted output, active low */
-#define LP_PINITP       0x04  /* unchanged output, active low */
-#define LP_PSTROBE      0x01  /* short high output on raising edge */
-
-#define DELAY 0x10c6ul
-
-void lp_wait(ulong xloops)
-{
-	int d0;
-	__asm__("mull %0"
-		:"=d" (xloops), "=&a" (d0)
-		:"1" (xloops),"0" (current_cpu_data.loops_per_sec));
-	__delay(xloops);
-}
-static void __delay(ulong loops)
-{
-	int d0;
-	__asm__ __volatile__(
-		"\tjmp 1f\n"
-		".align 16\n"
-		"1:\tjmp 2f\n"
-		".align 16\n"
-		"2:\tdecl %0\n\tjns 2b"
-		:"=&a" (d0)
-		:"0" (loops));
-}
-
-put_lp(char c, short port)
-{
-	unsigned char status;
-
-	/* Wait for printer to be ready */
-	while (1) {
-		status = inb(STATUS(port));
-		if (status & LP_PERRORP) {
-			if (status & LP_PBUSY) {
-				break;
-			}
-		}
-	}
-
-	outb(d, DATA(c));
-	lp_wait(DELAY);
-	outb((LP_PSELECP | LP_PINITP | LP_PSTROBE), CONTROL(port));
-	lp_wait(DELAY);
-	outb((LP_PSELECP | LP_PINITP), CONTROL(port));
-	lp_wait(DELAY);
-}
-#endif
-
