@@ -1,4 +1,4 @@
-/* test.c - MemTest-86  Version 3.1
+/* test.c - MemTest-86  Version 3.2
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady
@@ -434,6 +434,162 @@ void movinv1(int iter, ulong p1, ulong p2)
 	}
 }
 
+/*
+ * Test all of memory using a "half moving inversions" algorithm using random
+ * numbers and their complment as the data pattern. Since we are not able to
+ * produce random numbers in reverse order testing is only done in the forward
+ * direction.
+ */
+void movinvr()
+{
+	int i, j, done, seed1, seed2;
+	volatile ulong *pe;
+	volatile ulong *start,*end;
+	ulong num;
+
+	/* Initialize memory with initial sequence of random numbers.  */
+	if (v->rdtsc) {
+		asm __volatile__ ("rdtsc":"=a" (seed1),"=d" (seed2));
+	} else {
+		seed1 = 521288629 + v->pass;
+		seed2 = 362436069 - v->pass;
+	}
+
+	/* Display the current seed */
+        hprint(LINE_PAT, COL_PAT, seed1);
+	rand_seed(seed1, seed2);
+	for (j=0; j<segs; j++) {
+		start = v->map[j].start;
+		end = v->map[j].end;
+		pe = start;
+		p = start;
+		done = 0;
+		do {
+			/* Check for overflow */
+			if (pe + SPINSZ > pe) {
+				pe += SPINSZ;
+			} else {
+				pe = end;
+			}
+			if (pe >= end) {
+				pe = end;
+				done++;
+			}
+			if (p == pe ) {
+				break;
+			}
+/* Original C code replaced with hand tuned assembly code */
+/*
+			for (; p < pe; p++) {
+				*p = rand();
+			}
+ */
+
+                        asm __volatile__ (
+                                "jmp L200\n\t"
+                                ".p2align 4,,7\n\t"
+                                "L200:\n\t"
+                                "call rand\n\t"
+				"movl %%eax,(%%edi)\n\t"
+                                "addl $4,%%edi\n\t"
+                                "cmpl %%ebx,%%edi\n\t"
+                                "jb L200\n\t"
+                                : "=D" (p)
+                                : "D" (p), "b" (pe)
+				: "eax"
+                        );
+
+			do_tick();
+			BAILR
+		} while (!done);
+	}
+
+	/* Do moving inversions test. Check for initial pattern and then
+	 * write the complement for each memory location. Test from bottom
+	 * up and then from the top down.  */
+	for (i=0; i<2; i++) {
+		rand_seed(seed1, seed2);
+		for (j=0; j<segs; j++) {
+			start = v->map[j].start;
+			end = v->map[j].end;
+			pe = start;
+			p = start;
+			done = 0;
+			do {
+				/* Check for overflow */
+				if (pe + SPINSZ > pe) {
+					pe += SPINSZ;
+				} else {
+					pe = end;
+				}
+				if (pe >= end) {
+					pe = end;
+					done++;
+				}
+				if (p == pe ) {
+					break;
+				}
+/* Original C code replaced with hand tuned assembly code */
+/*
+				for (; p < pe; p++) {
+					num = rand();
+					if (i) {
+						num = ~num;
+					}
+					if ((bad=*p) != num) {
+						error((ulong*)p, num, bad);
+					}
+					*p = ~num;
+				}
+*/
+				if (i) {
+					num = 0xffffffff;
+				} else {
+					num = 0;
+				}
+				asm __volatile__ (
+					"jmp L26\n\t" \
+
+					".p2align 4,,7\n\t" \
+					"L26:\n\t" \
+					"call rand\n\t"
+					"xorl %%ebx,%%eax\n\t" \
+					"movl (%%edi),%%ecx\n\t" \
+					"cmpl %%eax,%%ecx\n\t" \
+					"jne L23\n\t" \
+					"L25:\n\t" \
+					"movl $0xffffffff,%%edx\n\t" \
+					"xorl %%edx,%%eax\n\t" \
+					"movl %%eax,(%%edi)\n\t" \
+					"addl $4,%%edi\n\t" \
+					"cmpl %%esi,%%edi\n\t" \
+					"jb L26\n\t" \
+					"jmp L24\n" \
+
+					"L23:\n\t" \
+					"pushl %%esi\n\t" \
+					"pushl %%ecx\n\t" \
+					"pushl %%eax\n\t" \
+					"pushl %%edi\n\t" \
+					"call error\n\t" \
+					"popl %%edi\n\t" \
+					"popl %%eax\n\t" \
+					"popl %%ecx\n\t" \
+					"popl %%esi\n\t" \
+					"jmp L25\n" \
+
+					"L24:\n\t" \
+					: "=D" (p)
+					: "D" (p), "S" (pe), "b" (num)
+					: "eax", "ecx", "edx"
+				);
+				do_tick();
+				BAILR
+			} while (!done);
+		}
+	}
+}
+
 void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 {
 	int i, j, k=0, n = 0, done;
@@ -899,6 +1055,7 @@ void modtst(int offset, int iter, ulong p1, ulong p2)
 			BAILR
 		} while (!done);
 	}
+        cprint(LINE_PAT, COL_PAT, "          ");
 }
 
 /*
@@ -1118,6 +1275,7 @@ void block_move(int iter)
 /*
  * Test memory for bit fade.
  */
+#define STIME 5400
 void bit_fade()
 {
 	int j;
@@ -1125,6 +1283,9 @@ void bit_fade()
 	volatile ulong bad;
 	volatile ulong *start,*end;
 
+	test_ticks += (STIME * 2);
+	v->pass_ticks += (STIME * 2);
+
 	/* Do -1 and 0 patterns */
 	p1 = 0;
 	while (1) {
@@ -1145,7 +1306,7 @@ void bit_fade()
 			BAILR
 		}
 		/* Snooze for 90 minutes */
-		sleep (5400);
+		sleep (STIME);
 
 		/* Make sure that nothing changed while sleeping */
 		for (j=0; j<segs; j++) {
@@ -1269,11 +1430,9 @@ void ad_err2(ulong *adr, ulong bad)
 }
 void print_hdr(void)
 {
-	static int header = -1;
-	if (header == 1) {
+	if (v->ecount > 1) {
 		return;
 	}
-	header = 1;
 	cprint(LINE_HEADER, 0,  "Tst  Pass   Failing Address          Good       Bad     Err-Bits  Count Chan");
 	cprint(LINE_HEADER+1, 0,"---  ----  -----------------------  --------  --------  --------  ----- ----");
 }
@@ -1286,8 +1445,20 @@ static void update_err_counts(void)
 
 static void print_err_counts(void)
 {
+	int i;
+	char *pp;
+
 	dprint(LINE_INFO, COL_ERR, v->ecount, 6, 0);
 	dprint(LINE_INFO, COL_ECC_ERR, v->ecc_ecount, 6, 0);
+
+	/* Paint the error messages on the screen red to provide a vivid */
+	/* indicator that an error has occured */ 
+	if (v->msg_line < 24) {
+		for(i=0, pp=(char *)((SCREEN_ADR+v->msg_line*160+1));
+				 i<76; i++, pp+=2) {
+			*pp = 0x47;
+		}
+	}
 }
 
 static void common_err(ulong page, ulong offset)
@@ -1295,13 +1466,11 @@ static void common_err(ulong page, ulong offset)
 	ulong mb;
 
 	/* Check for keyboard input */
+	print_hdr();
 	check_input();
-
+	scroll();
 	print_err_counts();
 
-	print_hdr();
-
-	scroll();
 	mb = page >> 8;
 	dprint(v->msg_line, 0, v->test, 3, 0);
 	dprint(v->msg_line, 4, v->pass, 5, 0);
@@ -1337,7 +1506,9 @@ void print_err( ulong *adr, ulong good, ulong bad, ulong xor)
 void print_ecc_err(unsigned long page, unsigned long offset, 
 	int corrected, unsigned short syndrome, int channel)
 {
-	update_err_counts();
+	if (!corrected) {
+		update_err_counts();
+	}
 	++(v->ecc_ecount);
 	if (v->printmode == PRINTMODE_NONE) {
 		return;
@@ -1346,9 +1517,9 @@ void print_ecc_err(unsigned long page, unsigned long offset,
 
 	cprint(v->msg_line, 36, 
 		corrected?"corrected           ": "uncorrected         ");
-	hprint2(v->msg_line, 56, syndrome, 4);
-	cprint(v->msg_line, 66, "ECC"); 
-	dprint(v->msg_line, 71, channel, 2, 0);
+	hprint2(v->msg_line, 60, syndrome, 4);
+	cprint(v->msg_line, 68, "ECC"); 
+	dprint(v->msg_line, 74, channel, 2, 0);
 }
 
 #ifdef PARITY_MEM
@@ -1418,7 +1589,9 @@ void do_tick(void)
 	ulong h, l, t;
 
 	/* FIXME only print serial error messages from the tick handler */
-	print_err_counts();
+	if (v->ecount) {
+		print_err_counts();
+	}
 	
 	nticks++;
 	v->total_ticks++;
@@ -1479,7 +1652,7 @@ void do_tick(void)
 
 void sleep(int n)
 {
-	int i;
+	int i, ip=0;
 	ulong sh, sl, l, h, t;
 
 	/* save the starting time */
@@ -1508,12 +1681,16 @@ void sleep(int n)
 		i = t % 60;
 		dprint(LINE_TIME, COL_TIME+9, i%10, 1, 0);
 		dprint(LINE_TIME, COL_TIME+8, i/10, 1, 0);
+		if (i != ip) {
+			do_tick();
+			BAILR
+			ip = i;
+		}
 		t /= 60;
 		i = t % 60;
 		dprint(LINE_TIME, COL_TIME+6, i % 10, 1, 0);
 		dprint(LINE_TIME, COL_TIME+5, i / 10, 1, 0);
 		t /= 60;
 		dprint(LINE_TIME, COL_TIME, t, 4, 0);
-		BAILR
 	}
 }
